#!/usr/bin/python3
# PROJECT: Wavefront Alignments Algorithms 
# LICENCE: MIT License 
# AUTHOR(S): Santiago Marco-Sola <santiagomsola@gmail.com>
# DESCRIPTION: Compare alignment (*.alg) files
# USAGE: python3 wfa.alg.cmp.py -h

import sys
import re
import enum
import argparse

import numpy as np
import matplotlib
import matplotlib.pyplot as plt
from matplotlib.patches import Rectangle
from scipy.constants.constants import alpha

################################################################################
# Distances & penalties
################################################################################
class Distance(enum.Enum):
    edit = 1
    gap_linear = 2
    gap_affine = 3
    gap_affine_2p = 4
    
class Penalties:
    def __init__(self):
      self.distance = Distance.edit
      self.match = 0
      self.mismatch = 0
      self.indel = 0
      self.gap_open1 = 0
      self.gap_extend1 = 0
      self.gap_open2 = 0
      self.gap_extend2 = 0
      
def cigar_compute_score_edit(cigar_vector,penalties,ignore_misms):
  score = 0
  for op in cigar_vector:
    if op[1] in "DI": score += int(op[0])
    if op[1] in "X" and not ignore_misms: score += int(op[0])
  return score

def cigar_compute_score_linear(cigar_vector,penalties,ignore_misms):
  score = 0
  for op in cigar_vector:
    if op[1] == "M": score -= int(op[0]) * penalties.match
    if op[1] == "X" and not ignore_misms: score -= int(op[0]) * penalties.mismatch
    if op[1] in "DI": score -= int(op[0]) * penalties.indel
  return score

def cigar_compute_score_affine(cigar_vector,penalties,ignore_misms):
  score = 0
  for op in cigar_vector:
    if op[1] == "M": score -= int(op[0]) * penalties.match
    if op[1] == "X" and not ignore_misms: score -= int(op[0]) * penalties.mismatch
    if op[1] in "DI": score -= penalties.gap_open1 + int(op[0]) * penalties.gap_extend1
  return score

def cigar_compute_score_affine_2p(cigar_vector,penalties,ignore_misms):
  score = 0
  for op in cigar_vector:
    if op[1] == "M": score -= int(op[0]) * penalties.match
    if op[1] == "X" and not ignore_misms: score -= int(op[0]) * penalties.mismatch
    if op[1] in "DI": 
      score1 = penalties.gap_open1 + int(op[0]) * penalties.gap_extend1
      score2 = penalties.gap_open2 + int(op[0]) * penalties.gap_extend2
      score -= min(score1,score2)
  return score
      
def cigar_compute_score(cigar,penalties,ignore_misms):
  # Parse CIGAR
  #   cigar = "10M3D5M1I10M"
  #   cigar_vector = [('10', 'M'), ('3', 'D'), ('5', 'M'), ('1', 'I'), ('10', 'M')]
  cigar_vector = re.findall(r'(\d+)([MXDI])',cigar)
  # Evaluate CIGAR
  if penalties.distance == Distance.edit:
    return cigar_compute_score_edit(cigar_vector,penalties,ignore_misms)
  elif penalties.distance == Distance.gap_linear:
    return cigar_compute_score_linear(cigar_vector,penalties,ignore_misms)
  elif penalties.distance == Distance.gap_affine:
    return cigar_compute_score_affine(cigar_vector,penalties,ignore_misms)
  elif penalties.distance == Distance.gap_affine_2p:
    return cigar_compute_score_affine_2p(cigar_vector,penalties,ignore_misms)
  
################################################################################
# Alignment stats
################################################################################  
class AlignmentStats:
    def __init__(self):
      self.score_same = 0
      self.score_best1 = 0
      self.score_best2 = 0
      self.scores1 = list()
      self.scores2 = list()

    def __str__(self):
      total_alignments = float(self.score_same + self.score_best1 + self.score_best2)
      str = "[WFACompareAlignments::Stats]\n"
      str += "  => Alignments.common %d (%2.1f %%)\n" % (self.score_same,100.0*float(self.score_same)/total_alignments)
      str += "  => Alignments.best1  %d (%2.1f %%)\n" % (self.score_best1,100.0*float(self.score_best1)/total_alignments)
      str += "  => Alignments.best2  %d (%2.1f %%)"   % (self.score_best2,100.0*float(self.score_best2)/total_alignments)
      return str

    def __repr__(self):
      return __str__(self)

def plot_score_distribution(stats,input_path1,input_path2):
  # Plot
  matplotlib.use('Agg')
  # Draw length histogram
  fig,ax1 = plt.subplots()
  ax1.set_xlabel('Score')
  ax1.set_ylabel('Total Count')
  #   ax1.xaxis.grid(True)
  #   ax1.yaxis.grid(True)
  # Plot score histogram
  range_min = min(min(stats.scores1),min(stats.scores2))
  range_max = max(max(stats.scores1),max(stats.scores2))
  n, bins, patches = ax1.hist(stats.scores1,50,range=[range_min,range_max],color="royalblue",edgecolor='black',alpha=0.5) 
  n, bins, patches = ax1.hist(stats.scores2,50,range=[range_min,range_max],color="darkorange",edgecolor='black',alpha=0.5)
  start, end = ax1.get_xlim()
  ax1.set_xticks(np.arange(start,end,(end-start)/5))
  # Leyend
  handles = [Rectangle((0,0),1,1,color=c,ec="k") for c in ["royalblue","darkorange"]]
  labels= [input_path1,input_path2]
  plt.legend(handles,labels,loc="upper left")
  # Plot
  plt.title("Score distribution for '%s' and '%s'" % (input_path1,input_path2))
  fig.savefig("wfaCmpAlg." + input_path1 + "." + input_path2 + ".png",
              format='png',dpi=100,bbox_inches='tight')
  #plt.show()

################################################################################
# Compare both files
################################################################################
def compare_alignments(input_path1,input_path2,penalties,use_score,ignore_misms,verbose):
  # Alignment stats
  stats = AlignmentStats()  
  # Read both files and compare  
  input_file1 = open(input_path1,"rt")
  input_file2 = open(input_path2,"rt")
  # Read lines
  line_no = 0
  while True:
      # Read lines
      line_no += 1
      line1 = input_file1.readline()
      if not line1: break # End of file
      line2 = input_file2.readline()
      if not line2: 
        print("[WFACompareAlignments] Files unsynch")
        exit(-1)
      # Extract alignments
      fields1 = line1.split()
      fields2 = line2.split()
      if use_score:
        cigar1 = None
        cigar2 = None
        score1 = int(fields1[0]) if len(fields1)<=2 else int(fields1[2])
        score2 = int(fields2[0]) if len(fields2)<=2 else int(fields2[2])
      else:
        cigar1 = fields1[1] if len(fields1)<=2 else fields1[5]
        cigar2 = fields2[1] if len(fields2)<=2 else fields2[5]
        # Evaluate CIGAR's score
        score1 = cigar_compute_score(cigar1,penalties,ignore_misms)
        score2 = cigar_compute_score(cigar2,penalties,ignore_misms)
      # Update stats
      stats.scores1.append(score1)
      stats.scores2.append(score2)
      if score1 == score2:
        stats.score_same += 1
      elif score1 > score2:
        stats.score_best1 += 1
      else:
        stats.score_best2 += 1
      # Verbose
      if verbose and score1 != score2:
        print(">Failed::%d" % line_no)
        print("  s1=%d\t%s" % (score1,cigar1))
        print("  s2=%d\t%s" % (score2,cigar2))
  # Close files
  input_file1.close()
  input_file2.close()
  # return stats
  return stats
  
################################################################################
# Main
################################################################################
# Configure arguments
parser = argparse.ArgumentParser()
parser.add_argument('-i1','--input1',required=True,help='Input file1 (*.alg)')
parser.add_argument('-i2','--input2',required=True,help='Input file2 (*.alg)')
parser.add_argument('-e','--edit',
                    help='Score alignments using edit distance (--edit)')
parser.add_argument('-l','--gap-linear',
                    help='Score alignments using gap-linear distance (--gap-linear=M,X,I)')
parser.add_argument('-g','--gap-affine',
                    help='Score alignments using gap-affine distance (--gap-affine=M,X,O,E)')
parser.add_argument('-G','--gap-affine-2p',
                    help='Score alignments using gap-affine-2p distance (--gap-affine-2p=M,X,O1,E1,O2,E2)')
parser.add_argument('--use-score', action='store_true',default=False,
                    help='Compares the score provided in the file (default=use-cigar)')
parser.add_argument('--ignore-misms', action='store_true',default=False,
                    help='Ignores mismatches when computing the score (default=disable)')
parser.add_argument('-p','--plot', action='store_true',default=False,
                    help='Plots score distribution of both inputs (default=disable)')
parser.add_argument('-v','--verbose', action='store_true',default=False,
                    help='Display comparison differences (default=disable)')
parser.add_argument('-H',action='store_true',dest="human_readable",default=False)

# Parse arguments
args = parser.parse_args()

# Select distance
penalties = Penalties()
if args.edit is not None:
  pass # Default
elif args.gap_linear is not None:
  values = args.gap_linear.split(',')
  penalties.distance = Distance.gap_linear
  penalties.match = int(values[0])
  penalties.mismatch = int(values[1])
  penalties.indel = int(values[2])
elif args.gap_affine is not None:
  values = args.gap_affine.split(',')
  penalties.distance = Distance.gap_affine
  penalties.match = int(values[0])
  penalties.mismatch = int(values[1])
  penalties.gap_open1 = int(values[2])
  penalties.gap_extend1 = int(values[3])
elif args.gap_affine_2p is not None:
  values = args.gap_affine_2p.split(',')
  penalties.distance = Distance.gap_affine_2p
  penalties.match = int(values[0])
  penalties.mismatch = int(values[1])
  penalties.gap_open1 = int(values[2])
  penalties.gap_extend1 = int(values[3])
  penalties.gap_open2 = int(values[4])
  penalties.gap_extend2 = int(values[5])
else:
  print("[WFACompareAlignments] No distance provided. Using edit-distance (default)")
  
# Check penalties
if penalties.match > 0: 
  print("[WFACompareAlignments] Match penalty must be negative or zero")
  exit(-1)
if penalties.mismatch < 0 or \
   penalties.mismatch < 0 or \
   penalties.gap_open1 < 0 or \
   penalties.gap_extend1 < 0 or \
   penalties.gap_open2 < 0 or \
   penalties.gap_extend2 < 0:
  print("[WFACompareAlignments] Penalties must be positive or zero")
  exit(-1)
  
# Compare alignments from both files
stats = compare_alignments(
  args.input1,args.input2,penalties,
  args.use_score,args.ignore_misms,args.verbose)
print(stats) # Display stats

# Plot score distribution
if args.plot:
  plot_score_distribution(stats,args.input1,args.input2)
  
  


#!/usr/bin/python3
# PROJECT: Wavefront Alignments Algorithms 
# LICENCE: MIT License 
# AUTHOR(S): Santiago Marco-Sola <santiagomsola@gmail.com>
# DESCRIPTION: Rescore alignment (*.alg) files
# USAGE: python3 wfa.alg.rescore.py -h

import sys
import re
import enum
import argparse

import numpy as np
import matplotlib
import matplotlib.pyplot as plt
from matplotlib.patches import Rectangle
from scipy.constants.constants import alpha

################################################################################
# Distances & penalties
################################################################################
class Distance(enum.Enum):
    edit = 1
    gap_linear = 2
    gap_affine = 3
    gap_affine_2p = 4
    
class Penalties:
    def __init__(self):
      self.distance = Distance.edit
      self.match = 0
      self.mismatch = 0
      self.indel = 0
      self.gap_open1 = 0
      self.gap_extend1 = 0
      self.gap_open2 = 0
      self.gap_extend2 = 0
      
def cigar_compute_score_edit(cigar_vector,penalties):
  score = 0
  for op in cigar_vector:
    if op[1] in "DI": score += int(op[0])
    if op[1] in "X": score += int(op[0])
  return score

def cigar_compute_score_linear(cigar_vector,penalties):
  score = 0
  for op in cigar_vector:
    if op[1] == "M": score -= int(op[0]) * penalties.match
    if op[1] == "X": score -= int(op[0]) * penalties.mismatch
    if op[1] in "DI": score -= int(op[0]) * penalties.indel
  return score

def cigar_compute_score_affine(cigar_vector,penalties):
  score = 0
  for op in cigar_vector:
    if op[1] == "M": score -= int(op[0]) * penalties.match
    if op[1] == "X": score -= int(op[0]) * penalties.mismatch
    if op[1] in "DI": score -= penalties.gap_open1 + int(op[0]) * penalties.gap_extend1
  return score

def cigar_compute_score_affine_2p(cigar_vector,penalties):
  score = 0
  for op in cigar_vector:
    if op[1] == "M": score -= int(op[0]) * penalties.match
    if op[1] == "X": score -= int(op[0]) * penalties.mismatch
    if op[1] in "DI": 
      score1 = penalties.gap_open1 + int(op[0]) * penalties.gap_extend1
      score2 = penalties.gap_open2 + int(op[0]) * penalties.gap_extend2
      score -= min(score1,score2)
  return score
      
def cigar_compute_score(cigar,penalties):
  # Parse CIGAR
  #   cigar = "10M3D5M1I10M"
  #   cigar_vector = [('10', 'M'), ('3', 'D'), ('5', 'M'), ('1', 'I'), ('10', 'M')]
  cigar_vector = re.findall(r'(\d+)([MXDI])',cigar)
  # Evaluate CIGAR
  if penalties.distance == Distance.edit:
    return cigar_compute_score_edit(cigar_vector,penalties)
  elif penalties.distance == Distance.gap_linear:
    return cigar_compute_score_linear(cigar_vector,penalties)
  elif penalties.distance == Distance.gap_affine:
    return cigar_compute_score_affine(cigar_vector,penalties)
  elif penalties.distance == Distance.gap_affine_2p:
    return cigar_compute_score_affine_2p(cigar_vector,penalties)
  
################################################################################
# Compare both files
################################################################################
def rescore_alignments(input_path,output_path,penalties,verbose):
  # Open files
  input_file = open(input_path,"rt")
  output_file = open(output_path,"wt")
  # Read lines
  while True:
      # Read lines
      line = input_file.readline()
      if not line: break # End of file
      # Extract alignments
      fields = line.split()
      cigar = fields[1] if len(fields)<=2 else fields[5]
      # Evaluate CIGAR's score
      score = cigar_compute_score(cigar,penalties)
      # Print rescore and CIGAR 
      output_file.write("%d\t%s\n" % (score,cigar))
  # Close files
  input_file.close()
  output_file.close()
  
################################################################################
# Main
################################################################################
# Configure arguments
parser = argparse.ArgumentParser()
parser.add_argument('-i','--input',required=True,help='Input file (*.alg)')
parser.add_argument('-o','--output',required=True,help='Output file (*.alg)')
parser.add_argument('-e','--edit',
                    help='Score alignments using edit distance (--edit)')
parser.add_argument('-l','--gap-linear',
                    help='Score alignments using gap-linear distance (--gap-linear=M,X,I)')
parser.add_argument('-g','--gap-affine',
                    help='Score alignments using gap-affine distance (--gap-affine=M,X,O,E)')
parser.add_argument('-G','--gap-affine-2p',
                    help='Score alignments using gap-affine-2p distance (--gap-affine=M,X,O1,E1,O2,E2)')
parser.add_argument('-v','--verbose',action='store_true',default=False,
                    help='Display comparison differences (default=disable)')
parser.add_argument('-H',action='store_true',dest="human_readable",default=False)

# Parse arguments
args = parser.parse_args()

# Select distance
penalties = Penalties()
if args.edit is not None:
  pass # Default
elif args.gap_linear is not None:
  values = args.gap_linear.split(',')
  penalties.distance = Distance.gap_linear
  penalties.match = int(values[0])
  penalties.mismatch = int(values[1])
  penalties.indel = int(values[2])
elif args.gap_affine is not None:
  values = args.gap_affine.split(',')
  penalties.distance = Distance.gap_affine
  penalties.match = int(values[0])
  penalties.mismatch = int(values[1])
  penalties.gap_open1 = int(values[2])
  penalties.gap_extend1 = int(values[3])
elif args.gap_affine_2p is not None:
  values = args.gap_affine_2p.split(',')
  penalties.distance = Distance.gap_affine_2p
  penalties.match = int(values[0])
  penalties.mismatch = int(values[1])
  penalties.gap_open1 = int(values[2])
  penalties.gap_extend1 = int(values[3])
  penalties.gap_open2 = int(values[4])
  penalties.gap_extend2 = int(values[5])
else:
  print("[WFAReScoreAlignments] No distance provided. Using edit-distance (default)");
  
# Check penalties
if penalties.match > 0: 
  print("[WFACompareAlignments] Match penalty must be negative or zero")
  exit(-1)
if penalties.mismatch < 0 or \
   penalties.mismatch < 0 or \
   penalties.gap_open1 < 0 or \
   penalties.gap_extend1 < 0 or \
   penalties.gap_open2 < 0 or \
   penalties.gap_extend2 < 0:
  print("[WFACompareAlignments] Penalties must be positive or zero")
  exit(-1)
  
# Rescore alignments
rescore_alignments(args.input,args.output,penalties,args.verbose)

  
  


#!/usr/bin/python3
# PROJECT: Wavefront Alignments Algorithms 
# LICENCE: MIT License 
# AUTHOR(S): Santiago Marco-Sola <santiagomsola@gmail.com>
# DESCRIPTION: Plot WFA alignment matrices
# USAGE: python3 wfa.plot.py -h

import sys
import copy
import glob
import os.path
import argparse
import warnings

import numpy as np
import matplotlib.pyplot as plt
import matplotlib.colors as colors
import matplotlib.ticker as mticker
import seaborn as sns

################################################################################
# WFA-file parsing
################################################################################
def wfa_parse_file(filename):
  # Log
  print('[Parsing]',end='',flush=True)
  # Parse
  wfa_info = {}
  with open(filename) as fp:
    line = fp.readline() # Read line
    while line:
      # Parse data
      if line[0] == '#':
        fields = line.split();
        if fields[1] == "Heatmap":
          matrix_name = fields[2]
          matrix_list = []
          matrix_list.append(fp.readline().strip().split(',')) 
          line = fp.readline()
          while line and line[0] != '#':
            matrix_list.append(line.strip().split(',')) 
            line = fp.readline()
          M = np.matrix(matrix_list).astype(int)
          M = np.where(M==-1,np.nan,M)
          wfa_info[matrix_name] = M
          # Parse next line
          if not line: break
          else: continue
        elif fields[1] == "List":  
          if len(fields) <= 3: 
            wfa_info[fields[2]] = None
          else: 
            CIGAR = np.matrix(fields[3].replace(","," ")).astype(int)
            wfa_info[fields[2]] = CIGAR
        else:    
          wfa_info[fields[1]] = fields[2]
      # Read next line
      line = fp.readline()
  # Process some values
  wfa_info["WFAMode"] = wfa_info["WFAMode"][1:-1] 
  wfa_info["Distance"] = wfa_info["Penalties"][1:-1].split(',')[0]
  # Return
  return wfa_info

################################################################################
# WFA-Heatmap plotting
################################################################################
def wfa_plot_ticks(ax,data,detailed):
  # Compute dimensions
  ylen = data.shape[0]
  xlen = data.shape[1]
  plen = int(wfa_info["PatternLength"])
  tlen = int(wfa_info["TextLength"])
  # Detailed mode
  if detailed:
    # Parameters
    pattern = wfa_info["Pattern"]
    text = wfa_info["Text"]
    # Hacky font size estimation
    label_size = 80.0/float(max(xlen,ylen))
    
    # Set Y-axis (major)
    y_ticks_loc = np.arange(-0.5,ylen,1)
    ax.yaxis.set_minor_locator(mticker.FixedLocator(y_ticks_loc))
    ax.set_yticks(y_ticks_loc)
    ax.set_yticklabels([])
    # Set Y-axis (minor)
    ax.set_yticks(np.arange(0,ylen,1),minor=True)
    ax.set_yticklabels([y for y in pattern],fontsize=label_size,fontweight='bold',minor=True)
    
    # Set X-axis (major)
    x_ticks_loc = np.arange(-0.5,xlen,1)
    ax.xaxis.set_minor_locator(mticker.FixedLocator(x_ticks_loc))
    ax.set_xticks(x_ticks_loc)
    ax.set_xticklabels([])
    # Set X-axis (minor)
    ax.set_xticks(np.arange(0,xlen,1),minor=True)
    ax.set_xticklabels([x for x in text],fontsize=label_size,fontweight='bold',minor=True)
    # Set X on top
    ax.xaxis.tick_top()
    
    # Hide minor dashes
    ax.tick_params(axis='both',which='minor',length=0) 
    # Gridlines (based on major)
    ax.grid(which='major',color="black",linestyle='-',linewidth=0.25)
  else:
    # X-ticks
    x_ticks = [i for i in range(0,xlen-1,xlen//5)]
    if x_ticks[-1]+5 < xlen-1: x_ticks.append(xlen-1)  
    else: x_ticks[-1] = xlen-1
    x_ratio = tlen/xlen
    x_ticks_labels = [int(i*x_ratio) for i in x_ticks]
    x_ticks_labels[-1] = tlen-1
    # Y-ticks
    y_ticks = [i for i in range(0,ylen-1,ylen//5)]
    if y_ticks[-1]+5 < ylen-1: y_ticks.append(ylen-1)  
    else: y_ticks[-1] = ylen-1
    y_ratio = plen/ylen
    y_ticks_labels = [int(i*y_ratio) for i in y_ticks]
    y_ticks_labels[-1] = plen-1
    # Set ticks
    ax.set_yticks(y_ticks)
    ax.set_yticklabels(y_ticks_labels,fontsize=6)
    ax.set_xticks(x_ticks)
    ax.set_xticklabels(x_ticks_labels,fontsize=6)
    # Set grid
    ax.grid(which='major',color="black",linestyle='-',linewidth=0.25)

def wfa_plot_wavefront(wfa_info,title,ax,data,detailed,
                       xlabel=False,ylabel=False):
  # Compute dimensions
  ylen = data.shape[0]
  xlen = data.shape[1]
  # Title
  ax.set_title(title,fontsize=10) 
  # Set ticks & grid
  wfa_plot_ticks(ax,data,detailed)
  # Set labels
  if xlabel: ax.set_xlabel('Text',fontsize=8)
  if ylabel: ax.set_ylabel('Pattern',fontsize=8)
  # Create colorbar
  #cmap = LinearSegmentedColormap.from_list('rg',['tab:green','yellow','tab:red'],N=256)
  cmap = copy.copy(plt.cm.jet)
  cmap.set_bad('whitesmoke')
  # Heatmap
  im = ax.imshow(data,cmap=cmap)
  # Detailed mode (Loop over data and create text annotations)
  if detailed:
    fontsize = 80.0/float(max(xlen,ylen))
    for i in range(len(data)):
      for j in range(len(data[i])):
        if data[i,j]>=0:
          ax.text(j,i,int(data[i,j]),ha="center",va="center",color='w',fontsize=fontsize,fontweight='bold')
  # Return
  return im

def wfa_plot_cigar(ax,wfa_info):
  # Parameters
  plen = int(wfa_info["PatternLength"])
  tlen = int(wfa_info["TextLength"])
  ylen = wfa_info["M"].shape[0]
  xlen = wfa_info["M"].shape[1]
  marker_size = 80.0/float(max(xlen,ylen))
  # Plot CIGAR series
  def wfa_plot_cigar_scaled(cigar,x_ratio,y_ratio,marker,color,label):
    x = np.floor(cigar[:,0].astype('float64') * x_ratio).astype('int') if x_ratio < 1.0 else cigar[:,0]
    y = np.floor(cigar[:,1].astype('float64') * y_ratio).astype('int') if y_ratio < 1.0 else cigar[:,1]
    ax.scatter([x],[y],marker=marker,color=color,s=marker_size,linewidths=0,label=label)
  # Compute dims
  x_ratio = xlen/tlen  
  y_ratio = ylen/plen
  # Fetch CIGAR
  cigar_m = wfa_info["CIGAR-M"]
  cigar_x = wfa_info["CIGAR-X"]
  cigar_i = wfa_info["CIGAR-I"]
  cigar_d = wfa_info["CIGAR-D"]
  # Plot CIGAR
  if (cigar_m is not None): wfa_plot_cigar_scaled(cigar_m,x_ratio,y_ratio,',','limegreen','match')
  if (cigar_x is not None): wfa_plot_cigar_scaled(cigar_x,x_ratio,y_ratio,'o','red','misms')
  if (cigar_i is not None): wfa_plot_cigar_scaled(cigar_i,x_ratio,y_ratio,'>','orange','ins')
  if (cigar_d is not None): wfa_plot_cigar_scaled(cigar_d,x_ratio,y_ratio,'v','blue','del')
  ax.legend(loc="upper right",prop={'size': 3}) 

def wfa_plot_xtra(wfa_info,title,ax,data,detailed, 
                  xlabel=False,ylabel=False): 
  # Title
  ax.set_title(title,fontsize=10) 
  # Set ticks & grid
  wfa_plot_ticks(ax,data,detailed)
  # Set labels
  if xlabel: ax.set_xlabel('Text',fontsize=8)
  if ylabel: ax.set_ylabel('Pattern',fontsize=8)
  # Create colorbar
  cmap_ext = colors.ListedColormap(['dodgerblue','darkred'])
  cmap_ext_bounds = [10,20]
  cmap_ext_norm = colors.BoundaryNorm(cmap_ext_bounds,cmap_ext.N)
  cmap_ext.set_bad('whitesmoke')
  # Heatmap
  im = ax.imshow(data,cmap=cmap_ext)
  # Plot CIGAR
  if 'A' in wfa_info["WFAMode"]: wfa_plot_cigar(ax,wfa_info)
  # Return
  return im
 
def wfa_plot(filename,wfa_info,dpi,mode,detailed):
  # Log
  print('[Plotting]',end='',flush=True)
  # Parameters
  plen = int(wfa_info["PatternLength"])
  tlen = int(wfa_info["TextLength"])
  ylen = wfa_info["M"].shape[0]
  xlen = wfa_info["M"].shape[1]
  # Create plot
  compact = (mode=="compact");
  extended = (mode=="extended"); 
  if compact: 
    fig, ax1 = plt.subplots(nrows=1,ncols=1,dpi=dpi,sharex=True)
    im1 = wfa_plot_wavefront(wfa_info,'M-Wavefront',ax1,wfa_info["M"],detailed,xlabel=True,ylabel=True)
  elif extended or wfa_info["Distance"]=="Edit":
    fig, (ax1,ax2) = plt.subplots(nrows=1,ncols=2,dpi=dpi,sharex=True)
    im1 = wfa_plot_wavefront(wfa_info,'M-Wavefront',ax1,wfa_info["M"],detailed,xlabel=True,ylabel=True)
    im3 = wfa_plot_xtra(wfa_info,'CIGAR',ax2,wfa_info["Extend"],detailed,xlabel=True)
  elif wfa_info["Distance"]=="GapLineal" or wfa_info["Distance"]=="GapAffine":
    fig, (ax1,ax2,ax3) = plt.subplots(nrows=1,ncols=3,dpi=dpi,sharex=True)
    im1 = wfa_plot_wavefront(wfa_info,'M-Wavefront',ax1,wfa_info["M"],detailed,xlabel=True,ylabel=True)
    im2 = wfa_plot_wavefront(wfa_info,'I1-Wavefront',ax2,wfa_info["I1"],detailed,xlabel=True)
    im3 = wfa_plot_wavefront(wfa_info,'D1-Wavefront',ax3,wfa_info["D1"],detailed,xlabel=True)
  elif wfa_info["Distance"]=="GapAffine2p":    
    fig, ((ax1,ax2,ax5),(ax3,ax4,ax6)) = plt.subplots(nrows=2,ncols=3,dpi=dpi,sharex=True)
    im1 = wfa_plot_wavefront(wfa_info,'M-Wavefront',ax1,wfa_info["M"],detailed,xlabel=True,ylabel=True)
    im2 = wfa_plot_wavefront(wfa_info,'I1-Wavefront',ax2,wfa_info["I1"],detailed,xlabel=True)
    im4 = wfa_plot_wavefront(wfa_info,'D1-Wavefront',ax4,wfa_info["D1"],detailed,xlabel=True)
    im5 = wfa_plot_wavefront(wfa_info,'I2-Wavefront',ax5,wfa_info["I2"],detailed,ylabel=True)
    im6 = wfa_plot_wavefront(wfa_info,'D2-Wavefront',ax6,wfa_info["D2"],detailed)
    im3 = wfa_plot_xtra(wfa_info,'CIGAR',ax3,wfa_info["Extend"],detailed)
  # Color bar
  if compact:
    p0 = ax1.get_position().get_points().flatten()
    p1 = p0
    pass
  elif extended or wfa_info["Distance"]=="Edit":
    p0 = ax1.get_position().get_points().flatten()
    p1 = ax2.get_position().get_points().flatten()
    pass
  elif wfa_info["Distance"]=="GapLineal" or wfa_info["Distance"]=="GapAffine":
    p0 = ax1.get_position().get_points().flatten()
    p1 = ax3.get_position().get_points().flatten()
  elif wfa_info["Distance"]=="GapAffine2p":    
    p0 = ax3.get_position().get_points().flatten()
    p1 = ax6.get_position().get_points().flatten()
  ax_cbar = fig.add_axes([p0[0],0,p1[2]-p0[0],0.025])
  ax_cbar.tick_params(labelsize=6) 
  plt.colorbar(im1,cax=ax_cbar,orientation='horizontal')
  # Title
  file = os.path.basename(filename).replace('.plot','')
  title = "WFA-Plot(%s) %s[%s]" % (file,wfa_info["Penalties"],wfa_info["WFAMode"])
  fig.suptitle(title,fontsize=10)
  plt.subplots_adjust(top=0.85)
  # Plot
  plt.savefig(filename.replace('.plot','.png'),bbox_inches='tight')

################################################################################
# Main
################################################################################
# Configure arguments
parser = argparse.ArgumentParser()
parser.add_argument('-i','--input',action='store',help='Input file (*.plot)')
parser.add_argument('--dpi',type=int,action='store',default=1000,help='Plot resolution (default=1000)') # More than 2000 is hard to handle
parser.add_argument('--mode',action='store',default='compact',help='Plot mode in {compact,extended,full}')
parser.add_argument('-d','--detailed',action='store_true',default=False,help='Plot score values and sequences')
parser.add_argument('-H',action='store_true',dest="human_readable",default=False)

# Parse arguments
args = parser.parse_args()

# Open input
if args.input:
  input_files = [args.input]
else:
  input_files = glob.glob("*.plot")
  print('[WFA2png] Searching all *.plot ( Found %d file%c)' % (len(input_files),'s' if len(input_files)>1 else ' '))
  
# Plot each WFA file
print('[WFA2png] Plotting at %d dpi' % (args.dpi))
idx = 0
for filename in input_files:
  print('[WFA2png] [#%d] Generating \'%s\' ' % (idx,filename),end='',flush=True)
  wfa_info = wfa_parse_file(filename)
  wfa_plot(filename,wfa_info,args.dpi,args.mode,args.detailed)
  print('[Done!]',)
  idx += 1


  
  
